from . import Notifier
from twilio.rest import Client


class TwilioNotifier(Notifier):
    """Notifier to send SMS notification to phone number via Twilio"""
    def __init__(self, message, account_sid, account_token, to_number, from_number):
        """
        Constructor
        :param message: The default message to use in the notification.
        :param account_sid: The Twilio account SID.
        :param account_token: The Twilio account token.
        :param to_number: The default number to send the notification to.
        :param from_number: The Twilio verified number from which the notification will be received.
        """
        super().__init__(message)
        self.message = message
        self.to_number = to_number
        self.from_number = from_number
        self.client = Client(account_sid, account_token)

    def notify(self, message=None, to_number=None):
        """
        Send the notification.
        :param message: The optional message to use for the notification. Default is the message from the constructor.
        :param to_number: The optional number to send to. Default is the number from the constructor.
        :return: None
        """
        if not message:
            message = self.message

        if not to_number:
            to_number = self.to_number

        twilio_message = self.client.messages.create(
            to=to_number,
            from_=self.from_number,
            body=message
        )
        print("Sent message with SID: {0}".format(twilio_message.sid))




__author__ = "Varun Agrawal"
__copyright__ = "Copyright 2017."
__license__ = "MIT"
__version__ = "1.0.1"
__maintainer__ = "Varun Agrawal"
__email__ = "varunagrawal@gatech.edu"
__status__ = "Production"


from herald import *



class Notifier:
    """
    Base notifier class.
    """
    def __init__(self, message):
        self.message = message

    def notify(self, *args, **kwargs):
        """
        Send the notification
        :param args:
        :param kwargs:
        :return: None
        """
        pass


class TerminalNotifier(Notifier):
    """
    Send the notifier to the terminal.
    """
    def __init__(self, message):
        """
        Constructor
        :param message: The default message to use in the notifier.
        """
        super().__init__(message)
        self.default_message = message

    def notify(self, message=None):
        """
        Send the notification.
        :param message: The message to send in the notification.
        :return: None
        """
        if message:
            self.message = message
        else:
            self.message = self.default_message

        print(self.message)



from . import Notifier
import os
import httplib2
import base64
from apiclient import discovery
from oauth2client import client
from oauth2client import tools
from oauth2client.file import Storage
from email.mime.text import MIMEText

class GmailNotifier(Notifier):
    """
    Herald notifier that uses GMail to send notifications
    """
    def __init__(self, message, subject="Herald Notification", cc=[], client_secret_file='client_secret.json'):
        """

        :param message: The default message to use in the notification.
        :param subject: The subject line of the email that will be sent to GMail.
        :param cc: List of emails to send the notification to as `cc`.
        :param client_secret_file: The location of the secret file which has the GMail access tokens.
        """
        super().__init__(message)
        self.message = message
        self.subject = subject
        self.cc = cc
        self.SCOPES = ['https://www.googleapis.com/auth/gmail.send', 'https://www.googleapis.com/auth/gmail.compose']
        self.CLIENT_SECRET_FILE = client_secret_file
        self.APPLICATION_NAME = 'Herald by Varun Agrawal'
        self.get_credentials()

    def get_credentials(self):
        """Gets valid user credentials from storage.

        If nothing has been stored, or if the stored credentials are invalid,
        the OAuth2 flow is completed to obtain the new credentials.
        """
        home_dir = os.path.expanduser('~')
        credential_dir = os.path.join(home_dir, '.credentials')
        if not os.path.exists(credential_dir):
            os.makedirs(credential_dir)
        credential_path = os.path.join(credential_dir,
                                       'herald-credentials.json')

        store = Storage(credential_path)
        self.credentials = store.get()
        if not self.credentials or self.credentials.invalid:
            flow = client.flow_from_clientsecrets(self.CLIENT_SECRET_FILE, self.SCOPES)
            flow.user_agent = self.APPLICATION_NAME
            self.credentials = tools.run_flow(flow, store, None)
            print('Storing credentials to ' + credential_path)

    def notify(self, message=None):
        """
        Send the notification.
        :param message: The message string to use in the notification. If empty, the default message is used.
        :return: None
        """
        if not message:
            message = self.message
        http = self.credentials.authorize(httplib2.Http())
        service = discovery.build('gmail', 'v1', http=http)

        mail = MIMEText(message)
        mail["subject"] = self.subject

        try:
            user = service.users().getProfile(userId="me").execute()
            mail["to"] = user["emailAddress"]
            mail["from"] = user["emailAddress"]
            mail["cc"] =",".join(self.cc)

            mail = {'raw': base64.urlsafe_b64encode(mail.as_string().encode()).decode()}
            message = service.users().messages().send(userId="me", body=mail).execute()
            if message:
                print("Notification sent!")
            else:
                raise ValueError("Error sending message")

        except (Exception,) as ex:
            print(ex)


from .notifiers.gmail import GmailNotifier


class Herald:
    """
    Herald context manager to send notifications
    """
    def __init__(self, notifier=GmailNotifier, message="Hear ye, hear ye!"):
        self.notifier = notifier
        self.message = message

    def __enter__(self):
        # Don't do anything
        pass

    def __exit__(self, exc_type, exc_val, exc_tb):\
        # TODO deal with exceptions
        self.notify()
        return False

    def notify(self, message=None):
        """
        Send the notification.
        :param message: The message to send in the notification.
        :return: None
        """
        if message:
            self.message = message
        self.notifier.notify(self.message)



"""
Programmable, extensible and easy to use notification system
"""
from setuptools import find_packages, setup
from codecs import open
from os import path
import herald
from pypandoc import convert

dependencies = ["google-api-python-client", "twilio", "oauth2client", "httplib2"]


def read_md(f): return convert(f, 'rst')


setup(
    name='herald-notify',
    version=herald.__version__,
    url='https://github.com/varunagrawal/herald',
    license=herald.__license__,
    author=herald.__author__,
    author_email=herald.__email__,
    description='Programmable, extensible and easy to use notification system',
    long_description=read_md("README.md"),
    packages=find_packages(exclude=['tests']),
    include_package_data=True,
    zip_safe=False,
    platforms='any',
    install_requires=dependencies,
    keywords="herald, notification, notify, extensible",
    classifiers=[
        # As from http://pypi.python.org/pypi?%3Aaction=list_classifiers
        # 'Development Status :: 1 - Planning',
        # 'Development Status :: 2 - Pre-Alpha',
        # 'Development Status :: 3 - Alpha',
        # 'Development Status :: 4 - Beta',
        'Development Status :: 5 - Production/Stable',
        # 'Development Status :: 6 - Mature',
        # 'Development Status :: 7 - Inactive',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: POSIX',
        'Operating System :: MacOS',
        'Operating System :: Unix',
        'Operating System :: Microsoft :: Windows',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
)